unit Sample12;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Blocks, Statsim, ComCtrls;

type
  TForm1 = class(TForm)
    Memo: TMemo;
    Button1: TButton;
    MultiRand: TMultiRand;
    Label1: TLabel;
    ProgressBar: TProgressBar;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    EditArrive: TEdit;
    EditServ: TEdit;
    EditSim: TEdit;
    Tabulator1: TTabulator;
    Entrance: TGenerator;
    Queue1: TQueue;
    Queue2: TQueue;
    Queue3: TQueue;
    CashDesk1: TServer;
    CashDesk2: TServer;
    CashDesk3: TServer;
    ExitDoor: TTerminator;
    Model: TModel;
    procedure Button1Click(Sender: TObject);
    procedure EditArriveExit(Sender: TObject);
    procedure EditServExit(Sender: TObject);
    procedure EditSimExit(Sender: TObject);
    procedure EntranceBeforeTimeGoOn(Sender: TAggregate);
    procedure EntranceExit(Sender: TBlock; Trans: TTransaction);
    procedure EntranceRouting(Sender: TBlock; Trans: TTransaction);
    procedure Queue1Routing(Sender: TBlock; Trans: TTransaction);
    procedure Queue2Routing(Sender: TBlock; Trans: TTransaction);
    procedure Queue3Routing(Sender: TBlock; Trans: TTransaction);
    procedure CashDesk1Enter(Sender: TBlock; Trans: TTransaction);
    procedure CashDesk1Routing(Sender: TBlock; Trans: TTransaction);
    procedure CashDesk2Enter(Sender: TBlock; Trans: TTransaction);
    procedure CashDesk2Routing(Sender: TBlock; Trans: TTransaction);
    procedure CashDesk3Enter(Sender: TBlock; Trans: TTransaction);
    procedure CashDesk3Routing(Sender: TBlock; Trans: TTransaction);
    procedure ExitDoorEnter(Sender: TBlock; Trans: TTransaction);
    procedure ModelNewTime(Sender: TAggregate; Trans: TTransaction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  MyTransaction = class(TTransaction)
  public
     BirthTime: real;
  end;

var
  Form1: TForm1;
  ArrivalTime: real;    // Avarage arrival time
  ServiceTime: real;    // Average service time
  LimitTime: real;      // Simulation time


implementation

{$R *.DFM}

procedure TForm1.Button1Click(Sender: TObject);
var i: byte;
begin
   { Checking all input paramters }
   try
      ArrivalTime:=StrToFloat(Trim(EditArrive.Text));
   except
      MessageDlg('Invalid arrival time',mtError,[mbOK],0);
      EditArrive.SetFocus;
      Exit;
   end;
   if ArrivalTime<=0.0 then
   begin
      MessageDlg('Invalid arrival time',mtError,[mbOK],0);
      EditArrive.SetFocus;
      Exit;
   end;

   try
      ServiceTime:=StrToFloat(Trim(EditServ.Text));
   except
      MessageDlg('Invalid service time',mtError,[mbOK],0);
      EditServ.SetFocus;
      Exit;
   end;
   if ServiceTime<=0.0 then
   begin
      MessageDlg('',mtError,[mbOK],0);
      EditServ.SetFocus;
      Exit;
   end;

   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditServ.SetFocus;
      Exit;
   end;

   Model.SetTransactionClass(MyTransaction);
   Model.Simulate(LimitTime);
   with Tabulator1 do
   begin
      Memo.Lines.Clear;
      Memo.Lines.Add('The histogram of time spent in the bank');
      Memo.Lines.Add('Below '+FormatFloat('000.00',LowerBound)+':     '+
                     IntToStr(Hits(0)));
      for i:=1 to Tabulator1.IntervalCount do
      begin
         Memo.Lines.Add(FormatFloat('000.00',LowerBound+Interval*(i-1))+' - '+
                        FormatFloat('000.00',LowerBound+Interval*(i))+'   '+
                        IntToStr(Hits(i)));
      end;
      Memo.Lines.Add('Upper '+FormatFloat('000.00',LowerBound+
                     Interval*IntervalCount)+':     '+
                     IntToStr(Hits(IntervalCount+1)));
      Memo.Lines.Add('___________________________________________');
      Memo.Lines.Add('Average time in the bank: '+
                     FormatFloat('0.00',Tabulator1.Mean));
      Memo.Lines.Add('Deviation of time in the bank: '+
                     FormatFloat('0.00',Tabulator1.Deviation));
   end;
   Model.Reset;
   Tabulator1.Reset;
end;


procedure TForm1.EditArriveExit(Sender: TObject);
begin
   try
      ArrivalTime:=StrToFloat(Trim(EditArrive.Text));
   except
      MessageDlg('Invalid arrival time',mtError,[mbOK],0);
      EditArrive.SetFocus;
      Exit;               ;
   end;
   if ArrivalTime<=0.0 then
   begin
      MessageDlg('Invalid arrival time',mtError,[mbOK],0);
      EditArrive.SetFocus;
   end;
end;

procedure TForm1.EditServExit(Sender: TObject);
begin
   try
      ServiceTime:=StrToFloat(Trim(EditServ.Text));
   except
      MessageDlg('Invalid service time',mtError,[mbOK],0);
      EditServ.SetFocus;
      Exit;
   end;
   if ServiceTime<=0.0 then
   begin
      MessageDlg('Invalid service time',mtError,[mbOK],0);
      EditServ.SetFocus;
   end;

end;

procedure TForm1.EditSimExit(Sender: TObject);
begin
   try
      LimitTime:=StrToFloat(Trim(EditSim.Text));
   except
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditSim.SetFocus;
      Exit;
   end;
   if LimitTime<=0.0 then
   begin
      MessageDlg('Invalid simulation time',mtError,[mbOK],0);
      EditServ.SetFocus;
   end;
end;

procedure TForm1.EntranceBeforeTimeGoOn(Sender: TAggregate);
begin
  Sender.NextTime(0.0);
end;

procedure TForm1.EntranceExit(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(ArrivalTime));
  (Trans as MyTransaction).BirthTime:=ModelTime;
end;

procedure TForm1.EntranceRouting(Sender: TBlock; Trans: TTransaction);
var QueueID: integer;
    MinCount: longint;
begin
  if (CashDesk1.Count=0) and (Queue1.Count=0) then
  begin
     Sender.PassTo(Queue1);
     Exit;
  end;
  if (CashDesk2.Count=0) and (Queue2.Count=0) then
  begin
     Sender.PassTo(Queue2);
     Exit;
  end;
  if (CashDesk3.Count=0) and (Queue3.Count=0) then
  begin
     Sender.PassTo(Queue3);
     Exit;
  end;

  { Choosing the queue with minimal length }
  MinCount:=Queue1.Count;
  QueueID:=1;

  if Queue2.Count<MinCount then
  begin
     MinCount:=Queue2.Count;
     QueueID:=2;
  end;

  if Queue3.Count<MinCount then
  begin
     QueueID:=3;
  end;

  case QueueID of
     1: Sender.PassTo(Queue1);
     2: Sender.PassTo(Queue2);
     3: Sender.PassTo(Queue3);
  end;
end;

procedure TForm1.Queue1Routing(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(CashDesk1);
end;

procedure TForm1.Queue2Routing(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(CashDesk2);
end;

procedure TForm1.Queue3Routing(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(CashDesk3);
end;

procedure TForm1.CashDesk1Enter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(ServiceTime));
end;

procedure TForm1.CashDesk1Routing(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;

procedure TForm1.CashDesk2Enter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(ServiceTime));
end;

procedure TForm1.CashDesk2Routing(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;

procedure TForm1.CashDesk3Enter(Sender: TBlock; Trans: TTransaction);
begin
  Sender.NextTime(MultiRand.Exponential(ServiceTime));
end;

procedure TForm1.CashDesk3Routing(Sender: TBlock; Trans: TTransaction);
begin
  Sender.PassTo(ExitDoor);
end;


procedure TForm1.ExitDoorEnter(Sender: TBlock; Trans: TTransaction);
begin
  Tabulator1.PutValue(ModelTime-(Trans as MyTransaction).BirthTime);
end;

procedure TForm1.ModelNewTime(Sender: TAggregate; Trans: TTransaction);
var  NewPos: integer;
     Ratio: real;
begin
   Ratio:=ModelTime/LimitTime;
   if Ratio>1.0 then Ratio:=1.0;
   NewPos:=Round(Ratio*100.0);
   with ProgressBar do
      if Position<>NewPos then Position:=NewPos;
end;

end.
